// Factory.hpp
#pragma once

#include "Utils.hpp"
#include <memory>		// auto_ptr
#include <map>			// map
#include <stdexcept>
#include <sstream>

// CreateError
class CreateError : public std::runtime_error
{
private:
	typedef std::runtime_error	base;
public:
	CreateError(const std::string& message) : base(message)
	{}
};

// Creator
template < typename Product, typename Key, typename Arguments >
class Creator
{
public:
	typedef Product		product_type;
	typedef Key			key_type;
	typedef Arguments	arguments_type;
public:
	virtual	~Creator()	{}
	virtual key_type	ID() const = 0;
	// 쐬ɎsƂCreateErrorthrow邱ƁB
	virtual std::auto_ptr<product_type>	create(const arguments_type& args) = 0;
};

// Factory
template < typename Creator, typename Compare >
class Factory
{
public:
	typedef Creator	creator_type;
	typedef typename Creator::product_type		product_type;
	typedef typename Creator::key_type			key_type;
	typedef typename Creator::arguments_type	arguments_type;

private:
	typedef std::map<key_type, creator_type*, Compare>	creator_map;
	creator_map	m_Creators;
public:
	Factory()
	{}
	virtual ~Factory()
	{
		for_all(m_Creators, select2nd(deleter()));
	}
	// Product 쐬B
	virtual std::auto_ptr<product_type>	createProduct(key_type key, const arguments_type& args)
	{
		creator_map::iterator i = m_Creators.find(key);
		if(i == m_Creators.end())
		{
			std::ostringstream oss;
			oss << "Key not found. : key=" << key;
			throw CreateError(oss.str());
		}
		return i->second->create(args);
	}
	// creator ̏L Factory ɈڂBauto_ptrɂĂ񂾂ALqʂ邾Ȃ̂ŁccB
	virtual void	registerCreator(creator_type* creator)
	{
		m_Creators.insert(creator_map::value_type(creator->ID(), creator));
	}
};
