// FactoryUsingXerces.hpp
#pragma once

#include "Factory.hpp"
#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/sax2/XMLReaderFactory.hpp>
#include <xercesc/sax2/SAX2XMLReader.hpp>
#include <xercesc/sax2/DefaultHandler.hpp>

// CreatorUsingXml : Key = const wchar_t*, Arguments = Attributes
template < typename Product >
class CreatorUsingXml : public Creator<Product, const wchar_t*, Attributes>
{
};

// FactoryUsingXml
template < typename Creator >
class FactoryUsingXml : public Factory<Creator, str::less<wchar_t> >, public DefaultHandler
{
private:
	typedef std::vector<product_type*>	ProductStack;
	ProductStack	m_Products;
public:
	FactoryUsingXml()
	{}
	virtual	~FactoryUsingXml()
	{
		for_all(m_Products, deleter());
	}
	virtual std::auto_ptr<product_type>	createFromXml(const char* xmlname)
	{
		clear();
		std::auto_ptr<SAX2XMLReader> parser(XMLReaderFactory::createXMLReader());

		parser->setContentHandler(this);
		parser->setErrorHandler(this);

		try
		{
			parser->parse(xmlname);
			if(m_Products.size() != 1)
			{
				clear();
				throw CreateError("Error in Construction");
			}
			std::auto_ptr<product_type> product(m_Products.back());
            m_Products.clear();
			return product;
		}
		catch(const XMLException& e)
		{
			std::ostringstream oss;
			oss << "XMLException : " << e.getMessage();
			clear();
			throw CreateError(oss.str());
		}
		catch(const SAXParseException& e)
		{
			std::ostringstream oss;
			oss << "SAXParseException : " << e.getMessage();
			clear();
			throw CreateError(oss.str());
		}
		catch(...)
		{
			clear();
			throw;
		}
	}

protected:
	// 쐬Ԃ̐eq֌Wݒ肷
	virtual void setMembership(product_type* parent, std::auto_ptr<product_type> child) = 0;

private:
	void	clear()
	{	// r Product ׂĔjB
		for_all(m_Products, deleter());
		m_Products.clear();
	}

	// SAX Handler methods
public:
	void startElement(
		const XMLCh* const	uri,
		const XMLCh* const	localname,
		const XMLCh* const	qname,
		const Attributes&	attrs)
	{
		std::auto_ptr<product_type> product = createProduct(qname, attrs);
		m_Products.push_back(product.release());
	}
	void endElement(
		const XMLCh* const	uri,
		const XMLCh* const	localname,
		const XMLCh* const	qname)
	{
		if(m_Products.size() <= 1)
		{	// ƊiDB߂ɉzIȐe΂肷̂ccH
			return;
		}
		std::auto_ptr<product_type> product(m_Products.back());
		m_Products.pop_back();
		setMembership(m_Products.back(), product);
	}
};
