// xerces-factory.cpp
#include "stdafx.h"

// TvFXMLEBWFbgKw쐬B

#include "FactoryUsingXerces.hpp"

#include <xercesc/util/PlatformUtils.hpp>
#ifdef NDEBUG
	#pragma comment(lib, "xerces-c_1.lib")
#else
	#pragma comment(lib, "xerces-c_1D.lib")
#endif

// Widget
class Widget
{
private:
	std::wstring	m_Name;
public:
	Widget(const std::wstring& name) : m_Name(name)
	{}
	virtual ~Widget()	{}
	virtual void	addChild(std::auto_ptr<Widget> child)
	{
		ASSERT(!"Widget::addChild() called. Must be implemented in a derived class.");
	}
	const std::wstring&	getName() const	{ return m_Name; }

	virtual void	print(std::ostream& os, int depth) = 0;
};

// WidgetCreator
typedef CreatorUsingXml<Widget>			WidgetCreator;

// WidgetFactory
class WidgetFactory : public FactoryUsingXml<WidgetCreator>
{
protected:
	virtual void setMembership(Widget* parent, std::auto_ptr<Widget> child)
	{
		parent->addChild(child);
	}
};

// TextLabel
class TextLabel : public Widget
{
	typedef Widget	base;
public:
	class Creator : public WidgetCreator
	{
	public:
		const wchar_t*	ID() const	{ return L"TextLabel"; }
		std::auto_ptr<Widget>	create(const Attributes& attr)
		{
			const wchar_t* name = attr.getValue(L"name");
			const wchar_t* text = attr.getValue(L"text");
			return auto_ptr_cast(new TextLabel(name, text));
		}
	};
private:
	std::wstring	m_Text;
public:
	TextLabel(const std::wstring& name, const std::wstring& text) : base(name), m_Text(text)
	{
	}
	const std::wstring&	getText() const	{ return m_Text; }
	virtual void	print(std::ostream& os, int depth)
	{
		for(int i = 0; i < depth; i++) os << '\t';
		os << "TextLabel : name=" << getName() << ", text=" << getText() << std::endl;
	}
};

// Window
class Window : public Widget
{
	typedef Widget	base;
public:
	class Creator : public WidgetCreator
	{
	public:
		const wchar_t*	ID() const	{ return L"Window"; }
		std::auto_ptr<Widget>	create(const Attributes& attr)
		{
			const wchar_t* name = attr.getValue(L"name");
			return auto_ptr_cast(new Window(name));
		}
	};
private:
	typedef std::vector<Widget*>	Children;
	Children	m_Children;
public:
	Window(const std::wstring& name) : base(name)
	{
	}
	virtual	~Window()
	{
		for_all(m_Children, deleter());
	}
	virtual void addChild(std::auto_ptr<Widget> child)
	{
		m_Children.push_back(child.release());
	}
	virtual void	print(std::ostream& os, int depth)
	{
		for(int i = 0; i < depth; i++) os << '\t';
		os << "Window : name=" << getName() << ", numChildren=" << m_Children.size() << std::endl;
		for(Children::iterator i = m_Children.begin(); i != m_Children.end(); ++i)
		{
			(*i)->print(os, depth+1);
		}
	}
};

WidgetFactory theFactory;

int main(int argc, char* argv[])
{
	using namespace std;
	setlocale(LC_ALL, "Japanese");	// wcstombs() 𐳏ɓ삳邽

	// XercesB
	XMLPlatformUtils::Initialize();
	// Xerces̏I\񂷂B
	atexit(XMLPlatformUtils::Terminate);

	// t@NgB
	theFactory.registerCreator(new TextLabel::Creator);
	theFactory.registerCreator(new Window::Creator);

	try
	{
		// XMLEBWFbgKw쐬B
		std::auto_ptr<Widget> w = theFactory.createFromXml("widgets.xml");
		// KȃIy[VB
		w->print(cout, 0);
	}
	catch(CreateError& e)
	{
		// XMLɖ肪炵B
		cerr << e.what() << endl;
	}
	return 0;
}
